#include <misc.h>
#include <params.h>

subroutine albsom (ncol, ocnfrac, icefrac, snowh, sicthk, &
                   coszrs, asdir, aldir, asdif, aldif)
!-----------------------------------------------------------------------
! Computes surface albedos over ocean for Slab Ocean Model (SOM)
!
! Two spectral surface albedos for direct (dir) and diffuse (dif)
! incident radiation are calculated. The spectral intervals are:
!   s (shortwave)  = 0.2-0.7 micro-meters
!   l (longwave)   = 0.7-5.0 micro-meters
!
! Uses knowledge of surface type to specify albedo, as follows:
!
! Ocean           Uses solar zenith angle to compute albedo for direct
!                 radiation; diffuse radiation values constant; albedo
!                 independent of spectral interval and other physical
!                 factors such as ocean surface wind speed.
!
! Ocean with      Ice albedos a function of ice thickness; snow overlying
!   sea ice       ice has solar zenith angle dependence. Total albedo over
!                 ice depends on amount of snow.
!
! Note, the code collects together ocean and ice surfaces for various
! computations in order to vectorize longitude loops.
!
! For more details , see Briegleb, Bruce P., 1992: Delta-Eddington
! Approximation for Solar Radiation in the NCAR Community Climate Model,
! Journal of Geophysical Research, Vol 97, D7, pp7603-7612).
!
!JR NOTE: this is only likely to work in a binary fractional configuration,
!         i.e. each grid box is entirely open ocean, sea ice, or land.
!
!---------------------------Code history--------------------------------
!
! Original version:         CCM1
! Standardized:             J. Rosinski, June 1992
! Reviewed:                 J. Kiehl, B. Briegleb, August 1992
! Rewritten for ocean only: J. Rosinski, May 1994
! Modified for SOM:         B. Briegleb  April 1995
! Reviewed:                 B. Briegleb  March 1996
! Reviewed:                 J. Kiehl     April 1996
!! Reviewed:                 B. Briegleb  May   1996
!
!-----------------------------------------------------------------------
!
! $Id: albsom.F90,v 1.1.4.2 2002/06/15 13:49:06 erik Exp $
!!! $Author: erik $
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid, only: pcols
   use oceanconst, only: snwmag, rghsnw

   implicit none
!-----------------------------------------------------------------------
#include <albedo.h>
!-----------------------------------------------------------------------
!
! Input arguments
!
   integer ncol

   real(r8) ocnfrac(pcols)       ! Land/ocean/seaice flag
   real(r8) icefrac(pcols)       ! Land/ocean/seaice flag
   real(r8) snowh(pcols)     ! Snow depth (liquid water equivalent)
   real(r8) sicthk(pcols)    ! Sea-ice thickness (m)
   real(r8) coszrs(pcols)    ! Cosine solar zenith angle
!
! Output arguments
!
   real(r8) asdir(pcols)     ! Srf alb for direct rad   0.2-0.7 micro-ms
   real(r8) aldir(pcols)     ! Srf alb for direct rad   0.7-5.0 micro-ms
   real(r8) asdif(pcols)     ! Srf alb for diffuse rad  0.2-0.7 micro-ms
   real(r8) aldif(pcols)     ! Srf alb for diffuse rad  0.7-5.0 micro-ms
!
!---------------------------Local variables-----------------------------
!
   integer i             ! Longitude index

   real(r8) frsnow           ! Horizontal fraction of snow cover
   real(r8) snwhgt           ! Physical snow height (m)
   real(r8) sasdir(pcols)    ! Snow alb for direct rad  0.2-0.7 micro-ms
   real(r8) saldir(pcols)    ! Snow alb for direct rad  0.7-5.0 micro-ms
   real(r8) sasdif(pcols)    ! Snow alb for diffuse rad  0.2-0.7 micro-ms
   real(r8) saldif(pcols)    ! Snow alb for diffuse rad  0.7-5.0 micro-ms
   real(r8), parameter :: adif = 0.06          ! Ocean albedo for diffuse radiation
!
! Statement functions
!
   logical ocean,seaice
   ocean(i)  = ocnfrac(i)>0.
   seaice(i) = icefrac(i)>0.
!
! Initialize all ocean/sea ice surface albedos to zero
!
   do i=1,ncol
      if (ocean(i) .or. seaice(i)) then
         asdir(i) = 0.
         aldir(i) = 0.
         asdif(i) = 0.
         aldif(i) = 0.
      endif
   enddo
!
!----------------------------- Sea Ice ---------------------------------
!
! Sea ice albedos for ice thickness less than .25m, or greater than .50m,
! are specified constants for each spectral interval; between these
! thicknesses, albedo linearly increases with sea ice thickness.
!
   do i=1,ncol
      if (seaice(i)) then
         if (coszrs(i) > 0.0) then
            asdir(i)  = sices
            aldir(i)  = sicel
            if (sicthk(i) <= .25) then
               asdir(i) = sicsmn
               aldir(i) = siclmn
            else if (sicthk(i) < .50) then
               asdir(i) = sicsmn + (sices-sicsmn)*(sicthk(i)-.25)/.25
               aldir(i) = siclmn + (sicel-siclmn)*(sicthk(i)-.25)/.25
            endif
            asdif(i)  = asdir(i)
            aldif(i)  = aldir(i)
            sasdif(i) = snws
            saldif(i) = snwl
         endif
      endif
   enddo
!
! Calculate zenith angle dependence for snow on sea ice
!
   do i=1,ncol
      if (seaice(i)) then
         if (snowh(i) > 0. .and. coszrs(i) > 0.) then
            if (coszrs(i) < 0.5) then
!
! Zenith angle regime 1 ( coszrs < 0.5 ).
! Set direct snow albedos (limit to 0.98 max)
!
               sasdir(i) = min (0.98_r8,sasdif(i) + (1. - sasdif(i))*0.5* &
                                ((3./(1. + 4.*coszrs(i))) - 1.))
               saldir(i) = min (0.98_r8,saldif(i) + (1. - saldif(i))*0.5* &
                                ((3./(1. + 4.*coszrs(i))) - 1.))
            else
!
! Zenith angle regime 2 ( coszrs >= 0.5 )
!
               sasdir(i) = snws
               saldir(i) = snwl
            endif
!
! Compute both diffuse and direct total ice albedos
!
            snwhgt   = snwmag*snowh(i)
            frsnow   = snwhgt/(rghsnw + snwhgt)
            asdir(i) = asdir(i)*(1. - frsnow) + sasdir(i)*frsnow
            aldir(i) = aldir(i)*(1. - frsnow) + saldir(i)*frsnow
            asdif(i) = asdif(i)*(1. - frsnow) + sasdif(i)*frsnow
            aldif(i) = aldif(i)*(1. - frsnow) + saldif(i)*frsnow
         endif
      endif
   enddo
!
!------------------------------ Ocean ----------------------------------
!
! Ice-free ocean albedos function of solar zenith angle only, and
! independent of spectral interval:
!
   do i=1,ncol
      if (ocean(i)) then
         if (coszrs(i) > 0.) then
            aldir(i) = (.026/(coszrs(i)**1.7 + .065)) + &
                       (.15*(coszrs(i) - 0.10)* &
                            (coszrs(i) - 0.50)* &
                            (coszrs(i) - 1.00)  )
            asdir(i) = aldir(i)
            aldif(i) = adif
            asdif(i) = adif
         endif
      endif
   enddo
!   
   return
end subroutine albsom
 
