#include <misc.h>
#include <params.h>

subroutine somoce (ztodt)
!-----------------------------------------------------------------------
! Slab Ocean Model (SOM) driver
!
!  Routine                     Task
!
!  somqad     Adjust ocean mixed layer Q flux under sea ice
!  srfoce     Calculate ocean/atmosphere latent/sensible heat
!               fluxes over open ocean; forecast ocean mixed 
!               layer depth temperature
!  srfsice    Calculate ocean/atmosphere latent/sensible heat
!               fluxes over sea ice; forecast ice temperatures
!               and snow/ice thickness changes
!  zenith     Solar zenith angles for albedos
!  albsom     Calculate ocean/ice albedos for next time step
!
!----------------------------Code History-------------------------------
!
! Original version:  B. Briegleb  April 1995
! Standardized:      J. Rosinski  October 1995
! Reviewed:          B. Briegleb  March 1996
! Reviewed:          J. Kiehl     April 1996
! Reviewed:          B. Briegleb  May   1996
!
!-----------------------------------------------------------------------
!
! $Id: somoce.F90,v 1.2.2.3 2002/06/15 13:49:08 erik Exp $
! $Author: erik $
!-----------------------------------------------------------------------
  use shr_kind_mod, only: r8 => shr_kind_r8
  use ppgrid,    only: pcols, begchunk, endchunk
  use comsrf, only: surface_state, surface_state2d, srfflx_parm, srfflx_parm2d, &
                    ocnfrac, icefrac, srfrpdel, prcsnw, sicthk, psm1, snowhice, fsns
  use phys_grid, only: get_ncols_p, get_rlat_all_p, get_rlon_all_p
  use ocean_data, only: qo, mld
  use oceanconst
  use time_manager, only: get_nstep, get_step_size, get_curr_calday

  implicit none

#include <comctl.h>
!
! Input/Output arguments
!
   real(r8) :: ztodt
!
! Local workspace
!
  integer c                       ! chunk index
  integer ncol                    ! number of columns
  integer nstep                   ! current time step
  integer dtime                   ! size of time step

  real(r8) :: clat(pcols)         ! current latitudes(radians)
  real(r8) :: clon(pcols)         ! current longitudes(radians)      
  real(r8) cosznext(pcols)        ! cosine solar zenith angle next timestep
  real(r8) cdaynext               ! calendar day for next timestep

  logical newsice(pcols,begchunk:endchunk) ! flag for ocean which forms ice this timestep
!-----------------------------------------------------------------------
!
! Calendar day for next time step
!
  nstep = get_nstep()
  dtime = get_step_size()
  cdaynext = get_curr_calday(offset=dtime)
!
! Adjust ocean mixed layer Q flux under sea ice in a globally
! conserving manner so that it is greater than a prescribed 
! hemispheric minimum.
!
  call t_startf ('somqad')
  call somqad (qo)
  call t_stopf ('somqad')

!
! Note: order of computation (open ocean first, then sea ice), is
! important. If any open ocean points form sea ice, they are flagged
! in the array 'newsice' to prevent recalculation in the ice routine.
!
!$OMP PARALLEL DO PRIVATE (C, NCOL, CLAT, CLON, COSZNEXT)

  do c=begchunk,endchunk
     ncol = get_ncols_p(c)
     call get_rlat_all_p (c, ncol, clat)
!
! Open ocean surface fluxes and temperatures
!
     call t_startf ('srfsomo')
     call srfsomo (ncol, icefrac(1,c), ocnfrac(1,c), newsice(1,c), ztodt, psm1(1,c), &
                   surface_state2d(c)%ubot, surface_state2d(c)%vbot, surface_state2d(c)%tbot, &
                      surface_state2d(c)%qbot, surface_state2d(c)%thbot, &
                   surface_state2d(c)%zbot, surface_state2d(c)%pbot, surface_state2d(c)%srfrad, &
                      surface_state2d(c)%tssub, srfrpdel(1,c), &
                   srfflx_parm2d(c)%cflx, srfflx_parm2d(c)%wsx, srfflx_parm2d(c)%wsy, &
                      srfflx_parm2d(c)%ts, srfflx_parm2d(c)%shf, &
                   srfflx_parm2d(c)%lhf, sicthk(1,c), mld(1,c), qo(1,c), clat(c), &
                   srfflx_parm2d(c)%lwup, srfflx_parm2d(c)%tref)
     call t_stopf ('srfsomo')
!
! Sea ice surface fluxes, temperatures, snow cover and thickness
!
! Note that "newsice" flag prevents computations from being done here when
! open ocean has changed to sea ice in 'srfsomo'.
!
     call t_startf ('srfsomi')
     call srfsomi (ncol, icefrac(1,c), newsice(1,c), ztodt, psm1(1,c), &
                   snowhice(1,c), surface_state2d(c)%ubot, surface_state2d(c)%vbot, &
                      surface_state2d(c)%tbot, surface_state2d(c)%qbot, &
                   surface_state2d(c)%thbot, surface_state2d(c)%zbot, surface_state2d(c)%pbot, &
                      surface_state2d(c)%srfrad, surface_state2d(c)%tssub, &
                   srfrpdel(1,c), srfflx_parm2d(c)%cflx, srfflx_parm2d(c)%wsx, &
                      srfflx_parm2d(c)%wsy, srfflx_parm2d(c)%ts,  &
                   srfflx_parm2d(c)%shf, srfflx_parm2d(c)%lhf, sicthk(1,c), &
                      fsns(1,c), prcsnw(1,c), &
                   mld(1,c), qo(1,c), clat(c), srfflx_parm2d(c)%lwup, srfflx_parm2d(c)%tref)
     call t_stopf ('srfsomi')
!
! Albedos for next time step 
!
     call get_rlon_all_p (c, ncol, clon)
     call zenith (cdaynext, clat, clon, cosznext, ncol)                               
     call albsom (ncol, ocnfrac(1,c), icefrac(1,c), snowhice(1,c), &
		    sicthk(1,c) , &
	          cosznext, srfflx_parm2d(c)%asdir, srfflx_parm2d(c)%aldir, &
                    srfflx_parm2d(c)%asdif, srfflx_parm2d(c)%aldif)
  end do

  return
end subroutine somoce
 
