#include <misc.h>
#include <params.h>

subroutine somqad (qo)
!-----------------------------------------------------------------------
! Slab Ocean Model (SOM) globally conserving adjustment of
! ocean mixed layer Q flux under sea ice.
!
! Adjust ocean mixed layer Q flux under sea ice if qo < qomin 
! in a globally conserving manner. The qo field is globally
! averaged; scanned for ice points; qo is set to be greater 
! than hemispheric values for such ice points; adjusted qo
! is then globally averaged again, and a constant flux is
! added over all ocean points to insure qo is globally
! conserved in the adjustment process.
!
! Diagnostic printout to unit 79 if desired.
!
!---------------------------Code history--------------------------------
!
! Original version:  Bruce P. Briegleb  April 1995
! Standardized:      J. Rosinski October 1995
! Reviewed:          B. Briegleb  March 1996
! Reviewed:          J. Kiehl     April 1996
! Reviewed:          B. Briegleb  May   1996
!
!-----------------------------------------------------------------------
!
! $Id: somqad.F90,v 1.2.2.1 2002/06/15 13:49:08 erik Exp $
! $Author: erik $
!
#if ( defined SPMD )
   use mpishorthand
#endif

   use shr_kind_mod, only: r8 => shr_kind_r8
   use time_manager, only: get_nstep, is_first_restart_step
   use ppgrid,       only: pcols, begchunk, endchunk
   use phys_grid,    only: gather_chunk_to_field, get_ncols_p
   use pmgrid,       only: plon, plat, masterproc
   use comsrf,       only: oro
   use commap,       only: w

   implicit none

!
! Arguments
!
   real(r8)  qo(pcols,begchunk:endchunk)    ! input ocean qflux
!
! Local workspace
!
   real(r8) :: oro_field(plon,plat)         ! global oro field
   real(r8) :: qo_field(plon,plat)          ! global qo field
   real(r8) glbinlat(plat)                  ! Accumulate multitasked partial sums
   real(r8) glbadlat(plat)                  ! Accumulate multitasked partial sums
   real(r8) sicfnhlat(plat)                 ! Accumulate multitasked partial sums
   real(r8) sicfshlat(plat)                 ! Accumulate multitasked partial sums
   real(r8) glbin                           ! input global mean over ocean
   real(r8) glbad                           ! adjusted global mean over ocean
   real(r8) sicfnh                          ! nh mean fractional area of sea ice
   real(r8) sicfsh                          ! sh mean fractional area of sea ice
   real(r8) wght                            ! area weight factor
   real(r8) wghthem                         ! area weight factor (diagnostic only)
   real(r8), parameter :: areahm = 255.0498 ! area of hemisphere in 10**6 km2 for printout
   real(r8), parameter :: qnhmin = -5.0     ! minimum qo allowed in north hem under sea ice
   real(r8), parameter :: qshmin = 30.0     ! minimum qo allowed in southern hem under sea ice
   real(r8) glbdiff                         ! qo difference for global conservation
!
   integer i, j                             ! lon, lat indices
   integer c                                ! chunk index
   integer ncol                             ! number of columns
   integer nstep                            ! current time step
!
   logical, parameter :: prnt = .true.      ! if true, printout diagnostics to unit 79
!
! Statement functions
!
   logical :: land, seaice
   real(r8) :: orog
   land(orog) = nint (orog) == 1
   seaice(orog) = nint (orog) == 2
!
! Master does all the work in computing global integral
!
   call gather_chunk_to_field (1, 1, 1, plon, qo, qo_field)
   call gather_chunk_to_field (1, 1, 1, plon, oro, oro_field)
!
! Calculate hemispheric fraction of sea ice for diagnostic purposes
! Compute input global mean qo over ocean
! Adjust input qo, setting >= to qnhmin,qshmin when sea-ice
! Compute adjusted global mean qo over ocean
!
   if (masterproc) then
      nstep = get_nstep()
!$OMP PARALLEL DO PRIVATE (I,J)
      do j=1,plat
         glbinlat(j)  = 0.
         glbadlat(j)  = 0.
         sicfshlat(j) = 0.
         sicfnhlat(j) = 0.

         do i=1,plon
            if (.not.land (oro_field(i,j))) then
               glbinlat(j) = glbinlat(j) + qo_field(i,j)*w(j)
            end if
            if (seaice (oro_field(i,j))) then
               if (j <= plat/2) then
                  qo_field(i,j) = max (qshmin, qo_field(i,j))
                  sicfshlat(j) = sicfshlat(j) + w(j)
               else
                  qo_field(i,j) = max (qnhmin, qo_field(i,j))
                  sicfnhlat(j) = sicfnhlat(j) + w(j)
               end if
            end if
            if (.not.land (oro_field(i,j))) then
               glbadlat(j) = glbadlat(j) + qo_field(i,j)*w(j)
            end if
         end do
      end do

      glbin = 0.
      glbad = 0.

      do j=1,plat
         glbin = glbin + glbinlat(j)
         glbad = glbad + glbadlat(j)
      end do
      wght = 0.0
      do j=1,plat
         do i=1,plon
            if (.not.land (oro_field(i,j))) then
               wght  = wght + w(j)
            end if
         end do
      end do

      glbin = glbin/wght
      glbad = glbad/wght
      glbdiff = glbin - glbad
!
! Print out diagnostics if desired
!
      if (prnt) then
         wghthem = 0.0
         sicfsh  = 0.0
         sicfnh  = 0.0
         do j=1,plat/2
            wghthem = wghthem + w(j)
         end do
         sicfsh = sicfsh/wghthem
         sicfnh = sicfnh/wghthem
         if( is_first_restart_step() ) then
            write(6,100) qnhmin,qshmin
100         format(' SOM adjusted qo '/' qnhmin = ',f6.2,' qshmin = ',f6.2/  &
                 '   nstep  sicanh  sicash  glbin   glbad  glbdiff')
         else
            write(6,200) nstep, sicfnh*areahm, sicfsh*areahm, glbin, glbad, glbdiff 
200         format(2x,i10,5(f7.3,1x))
         endif
      end if
   end if

#if ( defined SPMD )
   call mpibcast (glbdiff, 1, mpir8, 0, mpicom)
#endif
!
! Set output qo over all ocean points
!
   do c=begchunk,endchunk
      ncol = get_ncols_p(c)
      do i=1,ncol
         if (.not.land (oro(i,c))) then
            qo(i,c) = qo(i,c) + glbdiff
         end if
      end do
   end do

   return
end subroutine somqad
 
