#include <misc.h>
#include <params.h>

subroutine srfsomi (ncol    ,icefrac ,newsice ,ztodt   ,psm1    , &
                    snowh   ,ubot    ,vbot    ,tbot    ,qbot    , &
                    thbot   ,zbot    ,pmidm1  ,srfrad  ,tssub   , &
                    srfrpdel,qflx    ,taux    ,tauy    ,ts      , &
                    shflx   ,lhflx   ,sicthk  ,fsns    ,prcsnw  , &
                    ho      ,qo      ,clat    ,lwup    ,tref    )
!-----------------------------------------------------------------------
! Slab Ocean Model (SOM) sea ice calculation
!
! Calculates sea ice to atmosphere latent/sensible fluxes and stresses,
! then computes sea ice temperatures and snow/ice thickness changes.
! 
!---------------------------Code history--------------------------------
!
! Original version:      CCM1
! Standardized:          L. Buja, June 1992
! Reviewed:              B. Briegleb, J. Hack, August 1992
! Split by surface type: L. Bath, Dec 1994
! Modified for SOM:      B. Briegleb  April 1995
! Reviewed:              B. Briegleb  March 1996
! Reviewed:              J. Kiehl     April 1996
! Reviewed:              B. Briegleb  May   1996
!
!-----------------------------------------------------------------------
!
! $Id: srfsomi.F90,v 1.2.2.4 2002/11/22 02:11:16 eaton Exp $
! $Author: eaton $
!
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use ppgrid,  only: pcols
   use constituents, only: pcnst, pnats
   use comsrf,  only: plevmx

   implicit none

#include <comctl.h>
#include <comtsc.h>
!
! Input arguments
!
   integer ncol                  ! number of columns
   real(r8) oro(pcols)           ! Land/ocean/sea ice flag
   logical newsice(pcols)        ! Logical flag for brand new sea ice
   real(r8) ztodt                ! two times model timestep (2 delta-t)
   real(r8) psm1(pcols)          ! Surface pressure
   real(r8) snowh(pcols)         ! Snow depth (liquid water equivalent)
   real(r8) ubot(pcols)          ! Bottom level u wind
   real(r8) vbot(pcols)          ! Bottom level v wind
   real(r8) tbot(pcols)          ! Bottom level temperature
   real(r8) qbot(pcols)          ! Bottom level specific humidity
   real(r8) thbot(pcols)         ! Bottom level potential temperature
   real(r8) zbot(pcols)          ! Bottom level height above surface
   real(r8) pmidm1(pcols)        ! Bottom level pressure
   real(r8) srfrad(pcols)        ! Srf solar abs flux plus down longwave
   real(r8) srfrpdel(pcols)      ! 1./(pint(k+1)-pint(k))
   real(r8) ho(pcols)            ! Ocean mixed layer depth (m)
   real(r8) qo(pcols)            ! Ocean mixed layer qflux (J/m2/s)
   real(r8) clat                 ! latitude in radians
   real(r8) lwup(pcols)          ! Longwave up flux at srf
!
! Input/output arguments
!
   real(r8) tssub(pcols,plevmx)  ! Surface/sub-surface temperatures
!
! Output arguments
!
   real(r8), intent(out) :: qflx(pcols,pcnst+pnats)    ! Constituent flux (kg/m2/s)
   real(r8), intent(out) :: taux(pcols)          ! X surface stress (N/m2)
   real(r8), intent(out) :: tauy(pcols)          ! Y surface stress (N/m2)
   real(r8), intent(out) :: shflx(pcols)         ! Surface sensible heat flux (J/m2/s)
   real(r8), intent(out) :: lhflx(pcols)         ! Surface latent   heat flux (J/m2/s)
   real(r8), intent(inout) :: sicthk(pcols)        ! Sea-ice thickness (m)
   real(r8), intent(in) :: fsns(pcols)          ! Surface absorbed solar flux  (J/m2/s)
   real(r8), intent(in) :: prcsnw(pcols)        ! Total snow fall rate (liquid water equiv  m/s)
   real(r8), intent(out) :: ts(pcols)            ! Surface temperature (K)
!
!---------------------------Local variables-----------------------------
!
   integer indx(pcols)       ! Longitude index array (land)
   integer npts              ! Number of land points
   integer i,ii              ! Longitude indices
   integer m                 ! constituent index
   real(r8) dfntdt(pcols)        ! ts partial derivative of net srf flx
   real(r8) fnt(pcols)           ! Net surface flux for input conditions
   real(r8) ltheat(pcols)        ! Latent heat for given sfc conditions
   real(r8) tref(pcols)          ! Reference temperature
   real(r8) icefrac(pcols)       ! ice fraction
!
! Statement functions
!
   logical seaice
   seaice(i) = nint(oro(i)) == 2
!
! Find sea ice points that have not just changed to sea ice this timestep.
! Those new seaice points have already had their fluxes calculated for
! open ocean conditions.
!
   npts = 0
   do i=1,ncol
      if (seaice(i) .and. .not.newsice(i)) then
         npts = npts + 1
         indx(npts) = i
      end if
   end do
   if (npts == 0) return
! 
! Set latent heat for evaporation over sea ice:
!
   do ii=1,npts
      i = indx(ii)
      ltheat(i) = latvap + latice
   end do
!
! Compute surface fluxes, derviatives, and exchange coefficiants
!
   call flxsice (indx    ,npts    ,pmidm1  ,ubot    ,vbot    , &
                 tbot    ,qbot    ,thbot   ,zbot    ,srfrad  , &
                 tssub(1,1),ltheat,fnt     ,dfntdt  ,shflx   , &
                 lhflx   ,taux    ,tauy    ,tref    )
!
! Update sea-ice temperatures, thickness, and snow cover:
!
   call srftsbi (oro    ,indx   ,npts   ,fnt    ,dfntdt ,  &
                 ho     ,qo     ,prcsnw ,fsns   ,tssub  , &
                 sicthk ,snowh  ,icefrac,clat   )
!
! Evaluate ts and moisture flux
!
   do ii=1,npts
      i = indx(ii)
      ts(i) = tssub(i,1)
      lwup(i) = stebol * ts(i)**4
      qflx(i,1) = lhflx(i)/ltheat(i)
   end do
!
! Set non-water constituent fluxes to zero
!
   do m=2,pcnst+pnats
      do ii=1,npts
         i = indx(ii)
         qflx(i,m) = 0.
      end do
   end do

   return
end subroutine srfsomi
 
