#include <misc.h>
#include <params.h>

subroutine srfsomo (ncol    ,icefrac ,ocnfrac ,newsice ,ztodt   ,psm1    , &
                    ubot    ,vbot    ,tbot    ,qbot    ,thbot   , &
                    zbot    ,pmidm1  ,srfrad  ,tssub   ,srfrpdel, &
                    qflx    ,taux    ,tauy    ,ts      ,shflx   , &
                    lhflx   ,sicthk  ,ho      ,qo      ,clat    , &
                    lwup    ,tref    )
!-----------------------------------------------------------------------
! Slab Ocean Model (SOM) open ocean calculation
!
! Computes open ocean sensible/latent heat fluxes and stress components, 
! then forecasts ocean mixed layer temperature.
! 
!JR NOTE: this is only likely to work in a binary fractional configuration,
!         i.e. each grid box is entirely open ocean, sea ice, or land.
!
!---------------------------Code history--------------------------------
!
! Original version:  CCM1
! Standardized:      L. Buja, June 1992
! Reviewed:          B. Briegleb, J. Hack, August 1992
! Split by surface type:     L. Bath, Dec. 1994
! Modified for SOM:  B. Briegleb  April 1995
! Reviewed:          B. Briegleb  March 1996
! Reviewed:          J. Kiehl     April 1996
! Reviewed:          B. Briegleb  May   1996
!
!-----------------------------------------------------------------------
!
! $Id: srfsomo.F90,v 1.2.2.4 2002/11/22 02:11:16 eaton Exp $
! $Author: eaton $
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use oceanconst
   use ppgrid,       only: pcols
   use pmgrid,       only: plon
   use constituents, only: pcnst, pnats
   use comsrf,       only: plevmx
   use time_manager, only: get_step_size, get_nstep
   
   implicit none

#include <comtsc.h>
#include <comctl.h>
!
! Input arguments
!
   integer ncol                  ! number of columns
   real(r8) icefrac(pcols)       ! ice fraction
   real(r8) ocnfrac(pcols)       ! ocean fraction
   logical newsice(pcols)        ! Logical flag for brand new sea ice
   real(r8) ztodt                ! two times model timestep (2 delta-t)
   real(r8) psm1(pcols)          ! Surface pressure
   real(r8) ubot(pcols)          ! Bottom level u wind
   real(r8) vbot(pcols)          ! Bottom level v wind
   real(r8) tbot(pcols)          ! Bottom level temperature
   real(r8) qbot(pcols)          ! Bottom level specific humidity
   real(r8) thbot(pcols)         ! Bottom level potential temperature
   real(r8) zbot(pcols)          ! Bottom level height above surface
   real(r8) pmidm1(pcols)        ! Bottom level pressure
   real(r8) srfrad(pcols)        ! Srf solar abs flux plus down longwave
   real(r8) srfrpdel(pcols)      ! 1./(pint(k+1)-pint(k))
!
! Input/output arguments
!
   real(r8) tssub(pcols,plevmx)  ! Surface/sub-surface temperatures (K)
   real(r8) ho(pcols)            ! Ocean mixed layer depth (m)
   real(r8) qo(pcols)            ! Ocean mixed layer Q flux (J/m2/s)
   real(r8) clat                 ! Latitude in radians
   real(r8) lwup(pcols)          ! Longwave up flux at srf (J/m2/s)
   real(r8) tref(pcols)          ! Reference temperature
!
! Output arguments
!
   real(r8), intent(out) :: qflx(pcols,pcnst+pnats)    ! Constituent flux (kg/m2/s)
   real(r8), intent(out) :: taux(pcols)          ! X surface stress (N/m2)
   real(r8), intent(out) :: tauy(pcols)          ! Y surface stress (N/m2)
   real(r8), intent(out) :: ts(pcols)            ! surface temperature (K)
   real(r8), intent(out) :: shflx(pcols)         ! Surface sensible heat flux (J/m2/s)
   real(r8), intent(out) :: lhflx(pcols)         ! Surface latent   heat flux (J/m2/s)
   real(r8), intent(out) :: sicthk(pcols)        ! Sea-ice thickness (m)
!
!---------------------------Local variables-----------------------------
!
   integer i,ii             ! Longitude indices
   integer m                ! constituent index
   integer n                ! Sub-surface level index
   integer indx(pcols)      ! Longitude index array (land)
   integer npts             ! Number of land points
   integer dtime            ! size of time step
   integer nstep            ! current time step
   real(r8) fnt(pcols)          ! Net surface flux for input conditions
   real(r8) ltheat(pcols)       ! Latent heat for given sfc conditions
   real(r8) hi                  ! initial layer thickness of sea-ice
!
! For diagnostic prints
!
   logical prnt             ! If true, print out point information
   real(r8) rlon,rlat,dlon      ! longitude, latitude, longitude spacing
   real(r8) tsc                 ! final ts in degrees C
   integer iunout           ! integer fortran unit number for printout
   logical prnpnt
!
! Statement functions
!
   logical ocean,seaice
   ocean(i) = icefrac(i) > 0.
   seaice(i) = ocnfrac(i) > 0.
!
!-----------------------------------------------------------------------
!
! Find ocean surfaces
!
   npts = 0
   do i=1,ncol
!
! Initialize new sea ice flag
!
      newsice(i) = .false.
      if (ocean(i)) then
         npts = npts + 1
         indx(npts) = i
      end if
   end do

   if (npts > 0) then
! 
! Set latent heat for ocean surface
!
      do ii=1,npts
         i = indx(ii)
         ltheat(i) = latvap
      end do
!
! Compute surface fluxes, derviatives, and exchange coefficiants
!
      call flxoce (indx    ,npts    ,pmidm1  ,ubot    ,vbot    , &
                   tbot    ,qbot    ,thbot   ,zbot    ,tssub(1,1), &
                   ltheat  ,shflx   ,lhflx   ,taux    ,tauy    , &
                   lwup    ,tref    )
!
! Evaluate ts and moisture flux
!
      do ii=1,npts
         i = indx(ii)
         ts(i)     = tssub(i,1)
         qflx(i,1) = lhflx(i)/ltheat(i)
!
! Net surface flux at current surface temperature:
!
         fnt(i) = srfrad(i) - lwup(i) - shflx(i) - lhflx(i)
      end do
!
! Set non-water constituent fluxes to zero
!
      do m=2,pcnst+pnats
         do ii=1,npts
            i = indx(ii)
            qflx(i,m) = 0.
         end do
      end do
!
! Calculate open ocean change in ts:
!
      dlon = 360./plon
      dtime = get_step_size()
      do ii=1,npts
         i = indx(ii)
         ts(i) = ts(i) + ((fnt(i)+qo(i))/(rhoocn*cpocn*ho(i)))*dtime
         if (ts(i) < tof) then
!
! Sea-ice can form: compute thickness
!
            hi = rhoocn*cpocn*ho(i)*(tof-ts(i))/li
            if (hi > himin) then
               newsice(i) = .true.
               icefrac(i) = 1.
               ocnfrac(i) = 0.
               ts(i)      = tof
               sicthk(i)  = hi
            endif
         endif
!
! Update tssub:
!
         do n=1,plevmx
            tssub(i,n) = ts(i)              
         enddo
!
! Print out diagnostics
!
         prnt=.FALSE.
         if (prnt) then
            rlon = (i-1)*dlon
            rlat =  (180./3.1415926535)*clat
            if (rlon > 180.) rlon = rlon - 360.
            if (prnpnt(rlat,rlon,iunout)) then
               tsc = tssub(i,1) - tsm
!              nstep = get_nstep()
!              write(iunout,100) nstep,tsc,
!     $                          (tssub(i,4)-tsm),
!     $                          fnt(i),ho(i),qo(i),
!     $                          0.0,sicthk(i)
100            format(1x,i7,1x,2(f7.3,1x),f8.2,1x,4(f7.3,1x))
            endif
         end if
      end do
   end if

   return
end subroutine srfsomo
 
