#include <misc.h>
#include <params.h>

subroutine srftsbi (oro     ,indx    ,npts    ,fnt     ,dfntdt  , &
                    ho      ,qo      ,prcsnw  ,fsns    ,tssub   , &
                    sicthk  ,sicsnw  ,icefrac ,clat    )
!----------------------------------------------------------------------
! Slab Ocean Model (SOM) open ocean calculation
!
! Thermodynamic sea-ice model
! 
! Prognostics are:
!    (1) sea-ice thickness (m)
!    (2) sea-ice snow cover overlying ice (m liquid water equivalent)
!    (3) sea-ice tempertures in 4 evenly spaced layers (K)
!
! Sea-ice completely fills grid square (fractional coverage 1), and
! responds to thermal fluxes from atmosphere and those implied by
! an ocean mixed layer Q flux that maintains mixed layer temperature
! at freezing in the presence of sea-ice.
!
! Penetrating solar flux allowed for fraction of snow cover on sea-ice.
!
! Internal and boundary heat sources and fluxes are calculated, then
! backward implicit diffusion calculation is done over the 4 layers
! of sea-ice; if melting occurs sea-ice thickness and snow cover is
! adjusted, including allowing for sea-ice melt.
! 
!---------------------------Code history--------------------------------
!
! Original version:  CCM2
! Modified for SOM:  B. Briegleb  April 1995
! Reviewed:          B. Briegleb  March 1996
! Reveiwed:          J. Kiehl     April 1996
! Reviewed:          B. Briegleb  May   1996
!
!-----------------------------------------------------------------------
!
! $Id: srftsbi.F90,v 1.2.2.3 2002/06/15 13:49:09 erik Exp $
! $Author: erik $
!-----------------------------------------------------------------------
   use shr_kind_mod, only: r8 => shr_kind_r8
   use oceanconst
   use ppgrid,       only: pcols
   use pmgrid,       only: plon
   use comsrf,       only: plevmx 
   use time_manager, only: get_step_size

   implicit none

#include <comctl.h>
#include <comtsc.h>

   real(r8) oro(pcols)          ! Land/ocean/sea ice flag
   integer indx(pcols)      ! index of sea-ice surface types
   integer npts             ! number of points in indx
   real(r8) fnt(pcols)          ! net srf flux (updated on output)
   real(r8) dfntdt(pcols)       ! srfc tmp partial derivative net srf flx
   real(r8) ho(pcols)           ! ocean mixed layer thickness
   real(r8) qo(pcols)           ! heat flux from underlying ocean
   real(r8) prcsnw(pcols)       ! snow precipitation (liq wat equvlnt)
   real(r8) fsns(pcols)         ! srf solar abs flux
   real(r8) clat                ! latitude in radians
!
! Input/output arguments
!
   real(r8) tssub(pcols,plevmx) ! initial/final surface/sub-surface tmprs (K)
   real(r8) sicthk(pcols)       ! sea-ice thickness (m)
   real(r8) sicsnw(pcols)       ! sea-ice snow depth (liquid water equivalent m)
   real(r8) icefrac(pcols)      ! ice fraction
!
! Local workspace
!
   integer i,ii             ! longitude indices
   integer n,m              ! sub-surface level index
   integer jndx             ! surface type index
   integer ier              ! error return on tridiagonal solver
   integer dtime            ! size of time step
!
   integer nprt             ! for diagnostic printout
   real(r8) ts(pcols)           ! surface temperature
   real(r8) fbot(pcols)         ! thermal heat flux from ocean below sea-ice
   real(r8) fnet(pcols)         ! local net srf flux (over sea-ice, no solar)
   real(r8) z(pcols,0:plevmx)   !  level geometrical depth in m
   real(r8) cmass(pcols,plevmx) !  specific heat of soil in j/kg/k
   real(r8) rho(pcols,plevmx)   !  mass density (kg/m3)
   real(r8) tk(pcols,plevmx)    !  thermal conductivity (watts/m/k)
   real(r8) snwtop              !  snow physical depth for top layer
   integer, parameter :: nsrfty = 7       !  number of surface types
   real(r8) cmtype(nsrfty,plevmx)  !  mass specific heat (j/kg/k)
   real(r8) rhtype(nsrfty,plevmx)  !  mass density (kg/m3)
   real(r8) tktype(nsrfty,plevmx)  !  thermal conductivity (j/m/s)
!
   real(r8) frsnow(pcols)       ! fraction of surface covered with snow
   real(r8) snwhgt              ! physical depth of snow
!
! Properties of ice and air used to determine snow thermal properties
!
   real(r8), parameter :: rhice = 9.20e2  ! mass density of ice
   real(r8), parameter :: ksolar = 0.75   ! bulk solar abs coeff of sea-ice (/m)
   real(r8), parameter :: musol = 0.5     ! mean cos zen ang for solar rad (60 degrees)
   real(r8) tran(pcols,0:plevmx)   ! trans solar rad through sea-ice
   real(r8) htsrc(pcols,plevmx)    ! external heat source (w/m3)
   real(r8) diag(pcols,plevmx)     ! diagonal matrix elements
   real(r8) sbdiag(pcols,plevmx)   ! sub-diagonal matrix elements
   real(r8) spdiag(pcols,plevmx)   ! super-diagonal matrix elements
   real(r8) rhs(pcols,plevmx)      ! rhs of tri-diagonal matrix eqn.
   real(r8) ws(pcols*plevmx)       ! working storage for mtdlss
   real(r8) tin(pcols,plevmx)      ! initial sub-surface temperatures	
   real(r8) ti(pcols,plevmx)       ! sea-ice temperatures before final adj
   real(r8) to(plevmx)             ! sea-ice temperatures after  final adj
   real(r8) zi(0:plevmx)           ! thickness before final adjustment
   real(r8) zo(pcols,0:plevmx)     ! thickness before final adjustment
   real(r8) thck(pcols)            ! level thickness    (m)
   real(r8) cmty(pcols)            ! level mass heat capacity  (j/kg/k)
   real(r8) rhty(pcols)            ! level mass density        (kg/m3)
   real(r8) tkty(pcols)            ! level thermal conductivity (w/m/k)
   real(r8) rdtime     ! inverse model time step
   real(r8), parameter :: cmsnow = 2.09e3 ! snow mass heat capacity   (j/kg/k)
   real(r8), parameter :: rhsnow = 330.   ! snow mass density         (kg/m3)
   real(r8), parameter :: tksnow = 0.31   ! snow thermal conducitivity  (w/m/k)
   real(r8)  crt       ! cmass*rho*rdtime
   real(r8)  ztop      ! top level thickness
   real(r8)  rztop     ! 1/ztop
   real(r8)  tktop     ! top level bottom interface thermal conductivity
   real(r8)  zmpl      ! depth of top level bottom interface
   real(r8)  zm        ! mid depth of level
   real(r8)  delzpl    ! top level lower half thickness
   real(r8)  tmp       ! crt - dfntdt(i)*rztop
   real(r8)  delz      ! level thickness
   real(r8)  zmmn      ! level bottom interface depth
   real(r8)  delzmn    ! level bottom half thickness
   real(r8)  fpls      ! 1/(delz*delzpl)
   real(r8)  fmns      ! 1/(delz*delzmn)
   real(r8)  tkpls     ! level top half thermal conductivity
   real(r8)  tkmns     ! level bottom half thermal conductivity
   real(r8)  zbot      ! bottom level thickness
   real(r8)  tkbot     ! bottom level top interface thermal conductivity
   real(r8), parameter :: ls = 1.097e8  ! volumetric latent heat of fusion of snow  (j/m3)
   real(r8) tsc        ! final ts in degrees C
   real(r8) hi         ! initial layer thickness of sea-ice
   real(r8) hs         ! initial snow depth
   real(r8) hinew      ! new layer thickness of sea-ice
   real(r8) hinewm     ! new layer thickness of sea-ice when snow converted
   real(r8) etmelt     ! total surface layer energy available for snow/ice melt
   real(r8) esmelt     ! energy available for snow melt
   real(r8) hsmelt     ! snow melt
   real(r8) hsfall     ! snow fall
   real(r8) eimelt(plevmx)   ! layer energy for ice melt
   real(r8) hsadj            ! new adjusted snow cover depth
   real(r8) hiadj(plevmx)    ! new ice thickness (before final adjustment)
!
! For diagnostic prints
!
   logical prnt             ! If true, print out point information
   real(r8) rlon,rlat,dlon      ! longitude, latitude, longitude spacing
   integer iunout           ! integer fortran unit number for printout
   logical prnpnt
!
   data cmtype /4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2, &
                4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2, &
                4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2, &
                4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2/
!
   data rhtype /1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3, &
                1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3, &
                1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3, &
                1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3/
!
   data tktype /15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 , &
                15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 , &
                15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 , &
                15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 /
!
! Initialize 
!
   do n=1,plevmx
      do ii=1,npts
         i = indx(ii)
         tin(i,n) = tssub(i,n)
      end do
   end do
   dtime = get_step_size()
   rdtime = 1./dtime
!
! Compute internal heat source for sea-ice from absorbed solar radiation
!
! Compute transmission of solar radiation through sea-ice if no snow
! based on formula from maykut jgr 1982 87 7971-7984
!
   do ii=1,npts
      i = indx(ii)
      snwhgt    = snwmag*sicsnw(i)
      frsnow(i) = snwhgt/(rghsnw + snwhgt)
   end do
!
   do ii=1,npts
      i = indx(ii)
      tran(i,0) = 1.0
   end do
   do n=1,plevmx
      do ii=1,npts
         i = indx(ii)
         tran(i,n) = 0.805*exp(-(ksolar/musol)*((n*sicthk(i)/plevmx) - 0.1))
      end do
   end do
!
   do ii=1,npts
      i = indx(ii)
      do n=1,plevmx  
         htsrc(i,n) = fsns(i)*(1. - frsnow(i))*(tran(i,n-1)-tran(i,n)) / (sicthk(i)/plevmx)
      end do
   end do
!
! Define thermal properities for each sub/surface level, starting
! with the top level
!
   do ii=1,npts
      i = indx(ii)
      jndx    = 2
      thck(i) = sicthk(i)/plevmx
      cmty(i) = cmtype(jndx,1)
      rhty(i) = rhtype(jndx,1)
      tkty(i) = tktype(jndx,1)
   end do
!
   do ii=1,npts
      i = indx(ii)
!
! Initialize fields for no snow cover
!
      z(i,0)     = 0.0
      z(i,1)     = thck(i)
      cmass(i,1) = cmty(i)
      rho(i,1)   = rhty(i)
      tk(i,1)    = tkty(i)
!
! Modify level 1 fields when snow cover present.
! Weight by snow depth over sea-ice; assuming that the snow 
! overlies the sea-ice.
!
! Compute snow depth by multiplying by snwmag to convert
! liquid water equivalent into actual physcial snow depth
!
      snwtop    = snwmag*sicsnw(i)
      z(i,1)    = snwtop + thck(i)
      rho(i,1)  = (rhsnow*snwtop + rhty(i)*thck(i)) / z(i,1)
      cmass(i,1)= (rhsnow*snwtop*cmsnow + rhty(i)*thck(i)*cmty(i)) / & 
                  (rhsnow*snwtop + rhty(i)*thck(i))
      tk(i,1)   = (tksnow*tkty(i)*(snwtop + thck(i))) / &
                  (tksnow*thck(i) + tkty(i)*snwtop)
   end do
!
! Set surface thermal properties for the lower sub/surface levels:
!
   do n=2,plevmx
      do ii=1,npts
         i = indx(ii)
         jndx       = 2
         thck(i)    = sicthk(i)/plevmx
         cmass(i,n) = cmtype(jndx,n)
         rho(i,n)   = rhtype(jndx,n)
         tk(i,n)    = tktype(jndx,n)
      end do
      do ii=1,npts
         i = indx(ii)
         z(i,n)  = z(i,n-1) + thck(i)
      end do
   end do
!
! Define set of linear equations for temperature
! Compute heat flux from underlying ocean
!
   do ii=1,npts
      i = indx(ii)
      zbot    = 0.5*(z(i,plevmx) - z(i,plevmx-1))
      fbot(i) = -tk(i,plevmx)*(tof-tin(i,plevmx))/zbot
   end do
!
! Set up linear equations
!
   do ii=1,npts
      i = indx(ii)
      sbdiag(i,1)      = 0.
      spdiag(i,plevmx) = 0.
   end do
!
! Single level
!
   if (plevmx == 1) then
      do ii=1,npts
         i = indx(ii)
         rztop     = 1./(z(i,1) - z(i,0))
         crt       = (cmass(i,1)*rho(i,1)*rdtime)
         diag(i,1) = crt - dfntdt(i)*rztop
         rhs(i,1)  = diag(i,1)*tin(i,1) + fnt(i)*rztop - fbot(i)*rztop 
      end do
!
! More than one level
!
   else if (plevmx > 1) then
!
! Top level
!
      do ii=1,npts
         i = indx(ii)
         fnet(i) = fnt(i) - (fsns(i)*(1.-frsnow(i)))
      end do
!
      do ii=1,npts
         i = indx(ii)
         crt         = cmass(i,1)*rho(i,1)*rdtime
         ztop        = z(i,1) - z(i,0)
         rztop       = 1./ztop
         tktop       = 0.5*(tk(i,1) + tk(i,2))
         zmpl        = 0.5*(z(i,2) + z(i,1))
         zm          = 0.5*(z(i,1) + z(i,0))
         delzpl      = zmpl - zm
         fpls        = 1./(ztop*delzpl)
         tmp         = crt - dfntdt(i)*rztop
         diag(i,1)   = tmp + tktop*fpls
         spdiag(i,1) = -tktop*fpls
         rhs(i,1)    = tmp*tin(i,1) + fnet(i)*rztop + htsrc(i,1)
      end do
!
! Intermediate level
!
      do n=2,plevmx-1
         do ii=1,npts
            i = indx(ii)
            crt         = cmass(i,n)*rho(i,n)*rdtime
            delz        = z(i,n) - z(i,n-1)
            zmpl        = 0.5*(z(i,n+1) + z(i,n))
            zm          = 0.5*(z(i,n)   + z(i,n-1))
            zmmn        = 0.5*(z(i,n-1) + z(i,n-2))
            delzpl      = zmpl - zm
            delzmn      = zm - zmmn
            fpls        = 1./(delz*delzpl)
            fmns        = 1./(delz*delzmn)
            tkpls       = 0.5*(tk(i,n+1)+tk(i,n))
            tkmns       = 0.5*(tk(i,n)+tk(i,n-1))
            sbdiag(i,n) =  -tkmns*fmns
            diag(i,n)   =  crt + (tkpls*fpls + tkmns*fmns)
            spdiag(i,n) =  -tkpls*fpls
            rhs(i,n)    = crt*tin(i,n) + htsrc(i,n)
         end do
      end do
!
! Bottom level
!
      do ii=1,npts
         i = indx(ii)
         crt        = cmass(i,plevmx)*rho(i,plevmx)*rdtime
         zbot       = z(i,plevmx) - z(i,plevmx-1)
         zm         = 0.5*(z(i,plevmx)   + z(i,plevmx-1))
         zmmn       = 0.5*(z(i,plevmx-1) + z(i,plevmx-2))
         delzmn     = zm - zmmn
         tkbot      = 0.5*(tk(i,plevmx-1) + tk(i,plevmx))
         fmns       = 1./(zbot*delzmn)
         sbdiag(i,plevmx) = -tkbot*fmns
         diag(i,plevmx)   = crt + (tkbot*fmns)
         rhs(i,plevmx)    = crt*tin(i,plevmx) - fbot(i)/zbot + htsrc(i,plevmx)
      end do
   end if
!
! For the linear equation ax = b,  a and b arrays are now set; 
! solve for temperatures (x):
!
   if (plevmx == 1) then
      do ii=1,npts
         i = indx(ii)
         tssub(i,1) = rhs(i,1)/diag(i,1)
      end do
   else
      call mtdlss (sbdiag ,diag  ,spdiag ,rhs  ,tssub , &
                   plevmx ,pcols ,npts   ,indx ,ws    , &
                   pcols*plevmx  ,ier)
      if (ier /= 0) then
         write(6,*) 'srftsbi: error returned from mtdlss'
         call endrun ()
      end if
   end if
!
! Update net surface flux over all surfaces but open ocean
!
   do ii=1,npts
      i = indx(ii)
      fnt(i) = fnt(i) + dfntdt(i)*(tssub(i,1)-tin(i,1))
   end do
!
! Initialize output depths and temperatures:
!
   do ii=1,npts
      i = indx(ii)
      zo(i,0) = 0.0
      do n=1,plevmx
         zo(i,n) = z(i,n)
         ti(i,n) = tssub(i,n)
      end do
   end do
!
! Compute changes in sea-ice thickness and 
! accumulation or melting of snow
!
   do ii=1,npts
      i = indx(ii)
!
! Update surface temperature
!
      ts(i) = tssub(i,1)
!
! Set current snow depth and layer sea-ice thickness
!
      hs    = snwmag*sicsnw(i)
      hi    = sicthk(i)/plevmx
!
! Compute snow fall; only include snow fall for surface temperatures
! less than -0.1 Degrees Celcius
!
      if (ts(i) < tsm - 0.1) then
         hsfall = snwmag*prcsnw(i)*dtime
      else
         hsfall = 0.
      end if
!
! Compute energy available for snow melt
!
      if (ts(i) > tsm) then    
         etmelt     = cmass(i,1)*rho(i,1)*(ts(i)-tsm)*(z(i,1)-z(i,0))
         ts(i)      = tsm
         tssub(i,1) = tsm
      else
         etmelt = 0.
      end if
!
! Compute snow melt
!
      if (hs > 0.) then
         hsmelt = (etmelt/ls)
      else
         hsmelt = 0.
      end if
!
! Compute adjusted snow depth
!
      hsadj = hs + hsfall - hsmelt
!
! Compute actual snow melt energy and any available
! to melt sea-ice
!
      if (hsadj <= 0.) then
         hsadj     = 0.0
         hsmelt    = hs
         esmelt    = hs*ls
         eimelt(1) = etmelt - esmelt
      else
         esmelt    = 0.0
         eimelt(1) = 0.0
      end if
!
! Compute adjusted ice thickness:
!
      hiadj(1)    = hi - (eimelt(1)/li)
      do n=2,plevmx
         if (tssub(i,n) > tsm) then
            eimelt(n)  = cmass(i,n)*rho(i,n)*(tssub(i,n)-tsm)*(z(i,n)-z(i,n-1))
            hiadj(n)   = hi - (eimelt(n)/li)
            tssub(i,n) = tsm
         else
            eimelt(n)  = 0.0
            hiadj(n)   = hi
         end if
      end do
!
! Adjust lowest sea-ice layer
!
      hiadj(plevmx) = hiadj(plevmx) - ((fsns(i)*(1.-frsnow(i)) *tran(i,plevmx) + &
                      fbot(i) + qo(i))*(dtime/li))
!
! Compute adjusted total ice thickness per layer
!
      hinew = 0.0
      do n=1,plevmx
         hinew = hinew + hiadj(n)
      end do
!
! If sufficient snow on ice to suppress ice/snow 
! interface below sea-level, convert enough snow
! to sea-ice to bring interface back to sea-level,
! by using mass conservation and force balance.
!
! (.322 approximately  (rho ocean - rho ice ) / rho snow))
!
      if (hsadj > .322*hinew) then
!
! New total ice thickness when ice top at sea-level:
!
         hinewm   = ((rhsnow*hsadj + rhice*hinew)/rhoocn)
!
! Increase top layer ice thickness by new ice:
!
         hiadj(1) = hiadj(1) + (hinewm - hinew)
!
! Reset new ice thickness:
!
         hinew    = hinewm
!
! Calculate new snow thickness when ice top at sea-level
!
         hsadj    = ((rhoocn-rhice)/rhsnow)*hinew
      end if
!
! Formation of new open ocean
!
      if (hinew < himin) then
         ts(i) = tof - (ls*hsadj + li*hinew)/(rhoocn*cpocn*ho(i))
         do n=1,plevmx
            tssub(i,n) = ts(i)
            ti(i,n)    = tssub(i,n)
         end do
         sicthk(i)  = 0.
         sicsnw(i)  = 0.
         oro(i)     = 0.
         icefrac(i) = 0.
!
! Sea-ice remains
!
      else
         hinew = hinew/plevmx
!
! Interpolate temperatures for an evenly space sea-ice grid:
!
         zi(0)   = 0.0
         zo(i,0) = 0.0
         do n=1,plevmx
            ti(i,n) = tssub(i,n)
            zi(n)   = zi(n-1)   + hiadj(n)
            zo(i,n) = zo(i,n-1) + hinew
         end do
         zi(plevmx) = zo(i,plevmx)
!
! Index n output layer; index m input layers.
!
! Case descriptions as follows: 
!   (1) minimum zo in m layer but maximum zo not
!   (2) m layer contained completely in output n layer
!   (3) output n layer contained completely in m layer
!   (4) maximum zo in m layer but minimum zo not
!
         do n=1,plevmx
            to(n) = 0.0
            do m=1,plevmx
! Case 1
               if (zi(m-1) <= zo(i,n-1) .and. zo(i,n-1) < zi(m)) then
                  if (zo(i,n) > zi(m)) then
                     to(n) = to(n) + ti(i,m)*(zi(m)-zo(i,n-1))
                  end if
               end if
! Case 2
               if (zi(m-1) > zo(i,n-1) .and. zi(m) < zo(i,n)) then
                  to(n) = to(n) + ti(i,m)*(zi(m)-zi(m-1))
               end if
! Case 3
               if (zi(m-1) <= zo(i,n-1) .and. zo(i,n) <= zi(m)) then
                  to(n) = to(n) + ti(i,m)*(zo(i,n)-zo(i,n-1))
               end if
! Case 4
               if (zi(m-1) < zo(i,n) .and. zo(i,n) <= zi(m)) then
                  if (zo(i,n-1) < zi(m-1)) then
                     to(n) = to(n) + ti(i,m)*(zo(i,n)-zi(m-1))
                  end if
               end if
            end do
            to(n) = to(n)/(zo(i,n)-zo(i,n-1))
!
! If temperature change is close to machine roundoff, dont modify.
! The primary situation where this becomes important is when temperatures
! are within roundoff of melting.
!
            if (abs (to(n)-ti(i,n)) < 10.*epsilon (to)*to(n)) then
               to(n) = ti(i,n)
            end if
         end do
!
! Check for code error, and reset:
!
         do n=1,plevmx
            if (to(n) > 350.) then
               write(6,100) 
100            format('  n    ti      to        hiadj        zi         zo  ')
               do nprt=1,plevmx
                  write(6,110) nprt,ti(i,nprt),to(nprt), hiadj(nprt),zi(nprt),zo(i,nprt)
110               format(1x,i3,1x,5(1pe16.8,1x))
               end do
               to(n) = tssub(i,n)
            end if
!
            if (to(n) < 200.) then
               write(6,100) 
               do nprt=1,plevmx
                  write(6,110) nprt,ti(i,nprt),to(nprt), hiadj(nprt),zi(nprt),zo(i,nprt)
               end do
               to(n) = tssub(i,n)
            end if
         end do
         do n=1,plevmx
            tssub(i,n) = to(n)
            zo(i,n)    = zo(i,n) + hsadj
         end do
!
! Set output sea-ice thickness and snow cover:
!
         sicthk(i) = plevmx * hinew 
         sicsnw(i) = hsadj/snwmag
      end if
   end do
!
! Diagnostic prints
!
   prnt=.FALSE.
   if (prnt) then
      rlat   = (180./3.1415926535)*clat
      dlon   = 360./plon
      do ii=1,npts
         i = indx(ii)
         rlon = (i-1)*dlon
         if (rlon > 180.) rlon = rlon - 360.
         if (prnpnt (rlat,rlon,iunout)) then
            tsc = tssub(i,1) - tsm
!          nstep = get_nstep()
!          write(iunout,120) nstep,tsc,
!     $                      (tssub(i,4)-tsm),
!     $                      fnt(i),ho(i),qo(i),
!     $                      sicsnw(i),sicthk(i)
120         format(1x,i7,1x,2(f7.3,1x),f8.2,1x,4(f7.3,1x))
         end if
      end do
   end if

   return
end subroutine srftsbi
 
