module convert

  !----------------------------------------------------------------------- 
  ! Purpose: 
  ! read and writes initial data netCDF history files
  !----------------------------------------------------------------------- 

  use shr_kind_mod, only: r8 => shr_kind_r8
  use include
  implicit none

  ! Public methods

  public :: convert_filei

  ! Private methods

  private :: create_fileo

  private :: convert_ml_real
  private :: convert_sl_real
  private :: convert_sl_int

  SAVE

contains

  !=======================================================================

  subroutine convert_filei (fsurdat, finidati, finidato, cmdline)

    !----------------------------------------------------------------------- 
    ! Read initial data from netCDF instantaneous initial data history file 
    !-----------------------------------------------------------------------

    implicit none
    include 'netcdf.inc'

    ! ------------------------ arguments------ -----------------------------
    character(len=*), intent(in) :: fsurdat    !surface dataset
    character(len=*), intent(in) :: finidati   !input  initial dataset
    character(len=*), intent(in) :: finidato   !output initial dataset
    character(len=*), intent(in) :: cmdline    !command line arguments

    ! ------------------------ local variables -----------------------------
    integer :: i,j,k,l,m,n         !loop indices    
    integer :: ncidi               !netCDF dataset id
    integer :: ncido               !output net
    integer :: dimid               !netCDF dimension id 
    integer :: varid               !netCDF variable id
    integer :: dimlen              !input dimension length       
    integer :: ret                 !netcdf return code
    logical :: do_rtm               !true => initial conditions file has rtm data 
    logical :: do_dgvm              !true => initial conditions file has dgvm data
    ! --------------------------------------------------------------------

    write (6,*) ' Attempting to convert clm2.0 initial dataset'

    ! Open input initial conditions file

    call wrap_open (finidati, NF_NOWRITE, ncidi)

    ! Open new initial conditions output dataset 

    call wrap_create (finidato, NF_CLOBBER, ncido)

    ! Determine if initial file has RTM or DGVM data

    ret = nf_inq_varid (ncidi, 'RTMVOLR', varid)
    if (ret == NF_NOERR) then
       do_rtm = .true.
    else
       do_rtm = .false.
    endif

    ret = nf_inq_varid (ncidi, 'ITYPVEG_INI', varid)
    if (ret == NF_NOERR) then
       do_dgvm = .true.
    else
       do_dgvm = .false.
    endif

    ! Create new initial conditions output dataset

    call create_fileo (ncido, ncidi, cmdline, fsurdat, do_rtm, do_dgvm)

    ! Check consistency of input initial file dimensions and those calculated
    ! from surface dataset 

    ! Note: in clm2.0 - numland refers to the number of gridcells, whereas 
    ! in clm2.1 numland refers to the number of landunits

    call wrap_inq_dimid (ncidi, 'numland', dimid)
    call wrap_inq_dimlen (ncidi, dimid, dimlen)
    if (dimlen /= nland) then
       write (6,*) 'error: nland values disagree'
       write (6,*) 'finidat number of grid points = ',dimlen,' fsurdat number of grid points = ',nland
       stop
    end if

    call wrap_inq_dimid (ncidi, 'maxpatch', dimid)
    call wrap_inq_dimlen (ncidi, dimid, dimlen)
    if (dimlen /= maxpatch) then
       write (6,*) 'error: maxpatch values disagree'
       write (6,*) 'finidat maxpatch = ',dimlen,' fsurdat maxpatch = ',maxpatch
       stop
    end if

    call wrap_inq_dimid (ncidi, 'nlevsno', dimid)
    call wrap_inq_dimlen (ncidi, dimid, dimlen)
    if (dimlen /= nlevsno) then
       write (6,*) 'error: nlevsno values disagree'
       write (6,*) 'finidat levsno = ',dimlen,' fsurdat nlevsno = ',nlevsno
       stop
    end if

    call wrap_inq_dimid (ncidi, 'nlevsoi', dimid)
    call wrap_inq_dimlen (ncidi, dimid, dimlen)
    if (dimlen /= nlevsoi) then
       write (6,*) 'error: nlevsoi values disagree'
       write (6,*) 'finidat nlevsoi = ',dimlen,' fsurdat nlevsoi = ',nlevsoi
       stop
    end if

    call wrap_inq_dimid (ncidi, 'nlevtot', dimid)
    call wrap_inq_dimlen (ncidi, dimid, dimlen)
    if (dimlen /= nlevsoi+nlevsno) then
       write (6,*) 'error: nlevtot values disagree'
       write (6,*) 'finidat nlevtot = ',dimlen,' fsurdat nlevtot = ',nlevsno+nlevsoi
       stop 
    end if

    if (do_rtm) then
       call wrap_inq_dimid (ncidi, 'rtmlon', dimid)
       call wrap_inq_dimlen (ncidi, dimid, dimlen)
       if (dimlen /= rtmlon) then
          write (6,*) 'error: rtmlon values disagree'
          write (6,*) 'finidat rtmlon = ',dimlen,' fsurdat rtmlon = ',rtmlon
          stop
       end if
       call wrap_inq_dimid (ncidi, 'rtmlat', dimid)
       call wrap_inq_dimlen (ncidi, dimid, dimlen)
       if (dimlen /= rtmlat) then
          write (6,*) 'error: rtmlat values disagree'
          write (6,*) 'finidat rtmlat = ',dimlen,' fsurdat rtmlat = ',rtmlat
          stop
       end if
    endif

    ! Read/write  input initial data, convert to 1d form and write output initial data
    ! For the snow interfaces, are only examing the snow interfaces 
    ! above zi=0 which is why zisno and zsno have the same level dimension below

    call convert_ml_real('ZISNO_INI', ncidi, 'ZISNO', ncido, nlev=nlevsno)
                
    call convert_ml_real('ZSNO_INI', ncidi, 'ZSNO', ncido, nlev=nlevsno)
                
    call convert_ml_real('DZSNO_INI', ncidi, 'DZSNO', ncido, nlev=nlevsno)
                
    call convert_ml_real('H2OSOI_LIQ_INI', ncidi, 'H2OSOI_LIQ', ncido, nlev=nlevsno+nlevsoi)
                
    call convert_ml_real('H2OSOI_ICE_INI', ncidi, 'H2OSOI_ICE', ncido, nlev=nlevsno+nlevsoi)
                
    call convert_ml_real('T_SOISNO_INI', ncidi, 'T_SOISNO', ncido, nlev=nlevsno+nlevsoi)
                
    call convert_ml_real('T_LAKE_INI', ncidi, 'T_LAKE', ncido, nlev=nlevsoi)
                
    call convert_sl_real('T_VEG_INI', ncidi, 'T_VEG', ncido)
                
    call convert_sl_real('T_GRND_INI', ncidi, 'T_GRND', ncido)
                
    call convert_sl_real('H2OCAN_INI', ncidi, 'H2OCAN', ncido)
                
    call convert_sl_real('H2OSNO_INI', ncidi, 'H2OSNO', ncido)
                
    call convert_sl_real('SNOWDP_INI', ncidi, 'SNOWDP', ncido)
                
    call convert_sl_real('SNOWAGE_INI', ncidi, 'SNOWAGE', ncido)
                
    call convert_sl_int('SNLSNO_INI', ncidi, 'SNLSNO', ncido)

    if (do_rtm) then
       call wrap_inq_varid (ncidi, 'RTMVOLR', varid)
       call wrap_get_var_double (ncidi, varid, volr)
       call wrap_inq_varid (ncido, 'RTMVOLR', varid)
       call wrap_put_var_double (ncido, varid, volr)
    endif

    if (do_dgvm) then
       call convert_sl_int('ITYPVEG_INI', ncidi, 'ITYPVEG', ncido)

       call convert_sl_real('FPCGRID_INI', ncidi, 'FPCGRID', ncido)

       call convert_sl_real ('LAI_IND_INI', ncidi, 'LAI_IND', ncido)
                             
       call convert_sl_real ('CROWNAREA_INI', ncidi, 'CROWNAREA', ncido)
                             
       call convert_sl_real ('LITTERAG_INI', ncidi, 'LITTERAG', ncido)
                             
       call convert_sl_real ('LITTERBG_INI', ncidi, 'LITTERBG', ncido)
                             
       call convert_sl_real ('CPOOL_FAST_INI', ncidi, 'CPOOL_FAST', ncido)
                             
       call convert_sl_real ('CPOOL_SLOW_INI', ncidi, 'CPOOL_SLOW', ncido)
                             
       call convert_sl_real ('PRESENT_INI', ncidi, 'PRESENT', ncido)
                             
       call convert_sl_real ('NIND_INI', ncidi, 'NIND', ncido)
                             
       call convert_sl_real ('LM_IND_INI', ncidi, 'LM_IND', ncido)
                             
       call convert_sl_real ('SM_IND_INI', ncidi, 'SM_IND', ncido)
                             
       call convert_sl_real ('HM_IND_INI', ncidi, 'HM_IND', ncido)
                             
       call convert_sl_real ('RM_IND_INI', ncidi, 'RM_IND', ncido)
                             
       call convert_sl_real ('HTOP_INI', ncidi, 'HTOP', ncido)
                             
       call convert_sl_real ('WTXY_INI', ncidi, 'WTCOLXY', ncido)
    end if

    ! Close input and output files

    call wrap_close(ncidi)
    call wrap_close(ncido)

    write (6,*) ' Successfully converted clm2.0 initial dataset'

    return
  end subroutine convert_filei

  !=======================================================================

  subroutine create_fileo (ncido, ncidi, cmdline, fsurdat, do_rtm, do_dgvm)

    !----------------------------------------------------------------------- 
    ! Create output netCDF initial file dimensions, global attributes and 
    ! variable attributes
    !-----------------------------------------------------------------------

    implicit none
    include 'netcdf.inc'

    ! ------------------------ arguments------ -----------------------------
    integer, intent(in) :: ncido      !output nedCDF file id
    integer, intent(in) :: ncidi      !input nedCDF file id (for copy of attributes)
    character(len=*), intent(in) :: cmdline    !command line arguments
    character(len=*), intent(in) :: fsurdat    !surface dataset
    logical, intent(in) :: do_rtm     !true => input file has rtm data 
    logical, intent(in) :: do_dgvm    !true => input file has dgvm data 
    !-----------------------------------------------------------------------

    ! ------------------------ local variables ---------------------------
    integer :: n                      !index 
    integer :: dim1_id(1)             !netCDF dimension id for 1-d variables   
    integer :: dim2_id(2)             !netCDF dimension id for 2-d variables
    integer :: omode                  !netCDF dummy variable
    integer :: status                 !netCDF error status
    integer :: gridcell_dimid         !netCDF dimension id
    integer :: landunit_dimid         !netCDF dimension id
    integer :: column_dimid           !netCDF dimension id
    integer :: pft_dimid              !netCDF dimension id
    integer :: levsoi_dimid           !netCDF dimension id
    integer :: levsno_dimid           !netCDF dimension id
    integer :: levtot_dimid           !netCDF dimension id
    integer :: levlak_dimid           !netCDF dimension id
    integer :: rtmlon_dimid           !netCDF dimension id
    integer :: rtmlat_dimid           !netCDF dimension id
    integer :: varid                  !netCDF variable id
    integer :: ndimsi, nvarsi, ngattsi, unlimdimidi !temporaries
    integer :: attlen
    character(len=NF_MAX_NAME) :: attname
    character(len=500) :: text
    character(len= 80) :: file1, file2
    character(len= 80) :: arg
    ! --------------------------------------------------------------------

    ! set fill mode to "no fill" to optimize performance

    status = nf_set_fill (ncido, nf_nofill, omode)
    if (status /= nf_noerr) then
       write (6,*) ' netCDF error = ',nf_strerror(status)
       stop
    end if

    ! define output initial dataset dimensions 

    call wrap_def_dim (ncido, 'gridcell', numgrid, gridcell_dimid)
    call wrap_def_dim (ncido, 'landunit', numland, landunit_dimid)
    call wrap_def_dim (ncido, 'column'  , numcols, column_dimid)
    call wrap_def_dim (ncido, 'pft'     , numpfts, pft_dimid)
    call wrap_def_dim (ncido, 'levsno'  , nlevsno, levsno_dimid)
    call wrap_def_dim (ncido, 'levsoi'  , nlevsoi, levsoi_dimid)
    call wrap_def_dim (ncido, 'levlak'  , nlevlak, levlak_dimid)
    call wrap_def_dim (ncido, 'levtot'  , nlevsno+nlevsoi, levtot_dimid)
    call wrap_def_dim (ncido, 'rtmlon'  , rtmlon, rtmlon_dimid)
    call wrap_def_dim (ncido, 'rtmlat'  , rtmlat, rtmlat_dimid)

    ! define output single-level fields 

    dim1_id(1) = pft_dimid
    call wrap_def_var (ncido, 'T_VEG', nf_double, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'vegetation temperature')
    call wrap_put_att_text (ncido, varid, 'units', 'K')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim1_id(1) = column_dimid
    call wrap_def_var (ncido, 'T_GRND', nf_double, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'ground temperature')
    call wrap_put_att_text (ncido, varid, 'units', 'K')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim1_id(1) = pft_dimid
    call wrap_def_var (ncido, 'H2OCAN', nf_double, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'canopy water')
    call wrap_put_att_text (ncido, varid, 'units', 'kg/m2')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim1_id(1) = column_dimid
    call wrap_def_var (ncido, 'H2OSNO', nf_double, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'snow water')
    call wrap_put_att_text (ncido, varid, 'units', 'kg/m2')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim1_id(1) = column_dimid
    call wrap_def_var (ncido, 'SNOWDP', nf_double, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'snow depth')
    call wrap_put_att_text (ncido, varid, 'units', 'm')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim1_id(1) = column_dimid
    call wrap_def_var (ncido, 'SNOWAGE', nf_double, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'snow age')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim1_id(1) = column_dimid
    call wrap_def_var (ncido, 'SNLSNO', nf_int, 1, dim1_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'number of snow layers')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    ! define output multi-level fields

    dim2_id(1) = levtot_dimid
    dim2_id(2) = column_dimid
    call wrap_def_var (ncido, 'T_SOISNO', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'soil-snow temperature')
    call wrap_put_att_text (ncido, varid, 'units', 'K')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim2_id(1) = levlak_dimid
    dim2_id(2) = column_dimid
    call wrap_def_var (ncido, 'T_LAKE', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'lake temperature')
    call wrap_put_att_text (ncido, varid, 'units', 'K')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim2_id(1) = levtot_dimid
    dim2_id(2) = column_dimid
    call wrap_def_var (ncido, 'H2OSOI_LIQ', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'liquid water')
    call wrap_put_att_text (ncido, varid, 'units', 'kg/m2')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim2_id(1) = levtot_dimid 
    dim2_id(2) = column_dimid
    call wrap_def_var (ncido, 'H2OSOI_ICE', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'ice lens')
    call wrap_put_att_text (ncido, varid, 'units', 'kg/m2')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim2_id(1) = levsno_dimid
    dim2_id(2) = column_dimid
    call wrap_def_var (ncido, 'ZSNO', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'snow layer depth')
    call wrap_put_att_text (ncido, varid, 'units', 'm')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim2_id(1) = levsno_dimid
    dim2_id(2) = column_dimid
    call wrap_def_var (ncido, 'DZSNO', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'snow layer thickness')
    call wrap_put_att_text (ncido, varid, 'units', 'm')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    dim2_id(1) = levsno_dimid
    dim2_id(2) = column_dimid    
    call wrap_def_var (ncido, 'ZISNO', nf_double, 2, dim2_id, varid)
    call wrap_put_att_text (ncido, varid, 'long_name', 'snow interface depth')
    call wrap_put_att_text (ncido, varid, 'units', 'm')
    call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

    if (do_rtm) then
       dim2_id(1) = rtmlon_dimid 
       dim2_id(2) = rtmlat_dimid
       call wrap_def_var (ncido, 'RTMVOLR', nf_double, 2, dim2_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'water volumn in cell (volr)')
       call wrap_put_att_text (ncido, varid, 'units', 'm3')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')
    endif

    if (do_dgvm) then
       dim1_id(1) = pft_dimid

       call wrap_def_var (ncido, 'ITYPVEG', nf_int, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'plant functional type')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'FPCGRID', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', &
            'plant functional type cover - fraction of vegetated area')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'LAI_IND', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'LAI per individual')
       call wrap_put_att_text (ncido, varid, 'units', 'm2/m2')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'CROWNAREA', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'crown area')
       call wrap_put_att_text (ncido, varid, 'units', 'm2')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'LITTERAG', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'above ground litter')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C/m2 gridcell area')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')
       
       call wrap_def_var (ncido, 'LITTERBG', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'below ground litter')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C/m2 gridcell area')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'CPOOL_FAST', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'fast carbon pool')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C/m2 gridcell area')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'CPOOL_SLOW', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'slow carbon pool')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C/m2 gridcell area')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'PRESENT', nf_int, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'if pft is present is patch, 1=>yes,=>no')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'NIND', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'number of individuals')
       call wrap_put_att_text (ncido, varid, 'units', 'number of individuals/m2 gridcell area')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'LM_IND', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'individual leaf mass')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'SM_IND', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'individual stem mass')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'HM_IND', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'individual heartwood mass')
       call wrap_put_att_text (ncido, varid, 'units', 'grams C')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'RM_IND', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'individual root mass')
       call wrap_put_att_text (ncido, varid, 'units'     , 'grams C')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'HTOP', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'canopy top')
       call wrap_put_att_text (ncido, varid, 'units', 'm')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')

       call wrap_def_var (ncido, 'WTCOLXY', nf_double, 1, dim1_id, varid)
       call wrap_put_att_text (ncido, varid, 'long_name', 'column weight (relative to landunit)')
       call wrap_put_att_text (ncido, varid, 'cell_method', 'time: instantaneous')
    end if

    ! Copy global attributes from input initial file to output initial file
    
    call wrap_inq (ncidi, ndimsi, nvarsi, ngattsi, unlimdimidi)
    do n = 1,ngattsi
       call wrap_inq_attname (ncidi, NF_GLOBAL, n, attname)
       if (attname /= 'revision_id') then
          status = nf_inq_attlen (ncidi, NF_GLOBAL, attname, attlen)
          if (attlen > len(text)) then
             write(6,*) 'error: attribute ',trim(attname),' too long'
             stop 
          end if
          call wrap_copy_att (ncidi, NF_GLOBAL, attname, ncido, NF_GLOBAL)
       endif
    end do
    call addglobal (ncido, cmdline)

    ! Add surface dataset to global attributes

    call wrap_put_att_text (ncido, NF_GLOBAL, 'Surface_Dataset', trim(fsurdat))

    ! Finish creating out netCDF initial dataset (end define mode)

    status = nf_enddef(ncido)
    if (status /= NF_NOERR) call handle_error (status)

    return
  end subroutine create_fileo

  !=======================================================================

  subroutine convert_ml_real (varnamei, ncidi, varnameo, ncido, nlev)

    implicit none

    ! ------------------------ arguments ---------------------------------
    character(len=*), intent(in) :: varnamei    ! input variable name 
    integer , intent(in)  :: ncidi              ! input netCdf id
    character(len=*), intent(in) :: varnameo    ! output variable name
    integer , intent(in)  :: ncido              ! output netCDF id  
    integer , intent(in)  :: nlev               ! number of levels
    ! --------------------------------------------------------------------

    ! ------------------------ local variables --------------------------
    integer :: l,m,k,n                       !indices
    integer :: varid                         !variable id
    real(r8), allocatable :: rbufmli (:,:,:) !input array
    real(r8), allocatable :: rbufmlo (:,:)   !output array
    ! --------------------------------------------------------------------

    allocate (rbufmli(nland,maxpatch,nlev))
    allocate (rbufmlo(nlev,npatch))

    call wrap_inq_varid (ncidi, trim(varnamei), varid)
    call wrap_get_var_double(ncidi, varid, rbufmli)

    do m = 1, maxpatch              
       do l = 1, nland              
          if (landvec%wtxy(l,m) > 0.) then
             k = landvec%patch(l,m) 
             do n = 1,nlev
                rbufmlo(n,k) = rbufmli(l,m,n)
             end do
          endif
       end do
    end do

    call wrap_inq_varid (ncido, trim(varnameo), varid)
    call wrap_put_var_double(ncido, varid, rbufmlo)

    deallocate (rbufmli)
    deallocate (rbufmlo)

    return
  end subroutine convert_ml_real

  !=======================================================================

  subroutine convert_sl_real (varnamei, ncidi, varnameo, ncido)

    implicit none

    ! ------------------------ arguments ---------------------------------
    character(len=*), intent(in) :: varnamei    ! input variable name 
    integer , intent(in)  :: ncidi              ! input netCdf id
    character(len=*), intent(in) :: varnameo    ! output variable name
    integer , intent(in)  :: ncido              ! output netCDF id  
    ! --------------------------------------------------------------------

    ! ------------------------ local variables --------------------------
    integer :: l,m,k                       !indices
    integer :: varid                       !variable id
    real(r8), allocatable :: rbufsli (:,:) !input array
    real(r8), allocatable :: rbufslo (:)   !output array
    ! --------------------------------------------------------------------

    allocate (rbufsli(nland,maxpatch))
    allocate (rbufslo(npatch))

    call wrap_inq_varid (ncidi, trim(varnamei), varid)
    call wrap_get_var_double(ncidi, varid, rbufsli)

    do m = 1, maxpatch              
       do l = 1, nland            
          k = landvec%patch(l,m)    
          if (landvec%wtxy(l,m) > 0.) then
             rbufslo(k) = rbufsli(l,m)
          endif
       end do
    end do

    call wrap_inq_varid (ncido, trim(varnameo), varid)
    call wrap_put_var_double(ncido, varid, rbufslo)

    deallocate (rbufsli)
    deallocate (rbufslo)

    return
  end subroutine convert_sl_real

  !=======================================================================

  subroutine convert_sl_int (varnamei, ncidi, varnameo, ncido)

    implicit none
    include 'netcdf.inc'

    ! ------------------------ arguments ---------------------------------
    character(len=*), intent(in) :: varnamei
    integer , intent(in)  :: ncidi
    character(len=*), intent(in) :: varnameo
    integer , intent(in)  :: ncido
    ! --------------------------------------------------------------------

    ! ------------------------ local variables --------------------------
    integer :: l,m,k                  !indices
    integer :: varid                  !variable id
    integer, pointer :: ibufsli (:,:) !input array
    integer, pointer :: ibufslo (:)   !output array
    integer ret
    integer start(1), count(1)
    ! --------------------------------------------------------------------

    allocate (ibufsli(nland,maxpatch))
    allocate (ibufslo(npatch))

    call wrap_inq_varid (ncidi, trim(varnamei), varid)
    call wrap_get_var_int (ncidi, varid, ibufsli)

    do m = 1, maxpatch           
       do l = 1, nland         
          k = landvec%patch(l,m) 
          if (landvec%wtxy(l,m) > 0.) then
             ibufslo(k) = ibufsli(l,m)
          endif
       end do
    end do

    call wrap_inq_varid (ncido, trim(varnameo), varid)
    call wrap_put_var_int (ncido, varid, ibufslo)

    deallocate (ibufsli)
    deallocate (ibufslo)

    return
  end subroutine convert_sl_int

  !=======================================================================

end module convert
