subroutine surfmap (fsurdat)

!----------------------------------------------------------------------- 
! Read surface data file and make subgrid patches and determine
! 1d derived types needed for initial file read
! 
! Method: 
! The model's surface dataset recognizes 5 basic land cover types within 
! a grid cell: lake, wetland, urban, glacier, and vegetated. The vegetated 
! portion of the grid cell is comprised of up to [maxpatch_pft] PFTs 
! (patches). 
!
! The monthly surface dataset contains the following (on the 
! [lsmlon] x [lsmlat] grid). 
! o real edges of grid
! o real latitude  of grid cell (degrees)
! o real longitude of grid cell (degrees)
! o integer number of longitudes per latitude
! o integer surface type: 0 = ocean or 1 = land
! o integer soil color (1 to 9) for use with soil albedos
! o real soil texture, %sand, for thermal and hydraulic properties
! o real soil texture, %clay, for thermal and hydraulic properties
! o real % of cell covered by lake    for use as subgrid patch
! o real % of cell covered by wetland for use as subgrid patch
! o real % of cell that is urban      for use as subgrid patch
! o real % of cell that is glacier    for use as subgrid patch
! o integer PFTs
! o real % abundance PFTs (as a percent of vegetated area)
!-----------------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8
  use include
  implicit none
  include 'netcdf.inc'

  ! ------------------------ arguments -----------------------------------
  character(len=*) :: fsurdat            !input surface dataset

  ! ------------------------ local variables -----------------------------
  integer, parameter :: noveg = 0
  integer  :: i,j,k,l,m,n                !indices
  integer  :: li,lf                      !land indices
  integer  :: ncid, dimid, varid         !input netCDF id's
  integer  :: beg4d(4), len4d(4)         !netCDF variable edges
  real(r8) :: sumscl                     !temporory scalar sum 
  real(r8) :: sumwts                     !sum of wtxy over patches
  real(r8), allocatable :: pctgla(:,:)   !percent of grid cell that is glacier
  real(r8), allocatable :: pctlak(:,:)   !percent of grid cell that is lake
  real(r8), allocatable :: pctwet(:,:)   !percent of grid cell that is wetland
  real(r8), allocatable :: pcturb(:,:)   !percent of grid cell that is urbanized
  real(r8), allocatable :: sumvec(:,:)   !temporary vector sum 
  real(r8), allocatable :: pctpft(:,:,:) !percent of vegetated area for PFTs
  integer , allocatable :: pft(:,:,:)    !PFT 
  integer , allocatable :: veg(:,:,:)    !PFT 
  real(r8), allocatable :: wtxy(:,:,:)   !subgrid weights
  integer :: npatch_urban  !urban   patch number: 1 to maxpatch
  integer :: npatch_lake   !lake    patch number: 1 to maxpatch
  integer :: npatch_wet    !wetland patch number: 1 to maxpatch
  integer :: npatch_gla    !glacier patch number: 1 to maxpatch
  integer :: nvegpatch     !indices for pft cover types (clm2.1)
  ! ----------------------------------------------------------------------


  ! Obtain netcdf file and read surface data

  write (6,*) ' Attempting to read input surface boundary data .....'

  call wrap_open(fsurdat, 0, ncid)

  call wrap_inq_dimid  (ncid, 'lsmlon', dimid)
  call wrap_inq_dimlen (ncid, dimid, lsmlon)

  call wrap_inq_dimid  (ncid, 'lsmlat', dimid)
  call wrap_inq_dimlen (ncid, dimid, lsmlat)

  call wrap_inq_dimid  (ncid, 'nlevsoi', dimid)
  call wrap_inq_dimlen (ncid, dimid, nlevsoi)

  call wrap_inq_dimid  (ncid, 'lsmpft', dimid)
  call wrap_inq_dimlen (ncid, dimid, maxpatch_pft)

  if (maxpatch_pft == 10) then
     do_DGVM = .true.
  else if (maxpatch_pft == 4) then
     do_DGVM = .false.
  else
     write(6,*)'error: maxpatch_pft of ',maxpatch_pft,' not supported '
     stop
  endif
  npatch_urban = maxpatch_pft + 1 
  npatch_lake  = npatch_urban + 1 
  npatch_wet   = npatch_lake  + 1 
  npatch_gla   = npatch_wet   + 1 
  maxpatch     = npatch_gla       

  ! Set number of lake levels to number of soil levels

  nlevlak = nlevsoi 

  ! Allocate memory

  allocate (numlon(lsmlat))
  allocate (landmask(lsmlon,lsmlat))
  allocate (pctgla(lsmlon,lsmlat))   
  allocate (pctlak(lsmlon,lsmlat))   
  allocate (pctwet(lsmlon,lsmlat))   
  allocate (pcturb(lsmlon,lsmlat))   
  allocate (pctpft(lsmlon,lsmlat,maxpatch_pft))
  allocate (pft(lsmlon,lsmlat,maxpatch_pft))
  allocate (veg(lsmlon,lsmlat,maxpatch))
  allocate (wtxy(lsmlon,lsmlat,maxpatch))
  allocate (sumvec(lsmlon,lsmlat))

  ! Initialize surface data 

  landmask(:,:) = -999
  pctlak(:,:)   = -999.
  pctwet(:,:)   = -999.
  pcturb(:,:)   = -999.
  pctgla(:,:)   = -999.

  ! Read in variables

  call wrap_inq_varid (ncid, 'NUMLON', varid)
  call wrap_get_var_int   (ncid, varid  , numlon)

  call wrap_inq_varid (ncid, 'LANDMASK', varid) 
  call wrap_get_var_int (ncid, varid, landmask)

  call wrap_inq_varid (ncid, 'PCT_WETLAND', varid)
  call wrap_get_var_double (ncid, varid, pctwet)

  call wrap_inq_varid (ncid, 'PCT_LAKE', varid)
  call wrap_get_var_double (ncid, varid, pctlak)

  call wrap_inq_varid (ncid, 'PCT_GLACIER', varid)
  call wrap_get_var_double (ncid, varid, pctgla)

  call wrap_inq_varid (ncid, 'PCT_URBAN', varid)
  call wrap_get_var_double (ncid, varid, pcturb)

  call wrap_inq_varid (ncid, 'PFT', varid)
  call wrap_get_var_int (ncid, varid, pft)

  call wrap_inq_varid (ncid, 'PCT_PFT', varid)
  call wrap_get_var_double (ncid, varid, pctpft)

  call wrap_close(ncid)

  write (6,*) ' Finished reading input surface boundary data .....'

  ! Error check: valid PFTs and sum of cover must equal 100

  if (.not. do_DGVM) then
     sumvec(:,:) = abs(sum(pctpft,dim=3)-100.)
     do j=1,lsmlat
        do i=1,numlon(j)
           do m = 1, maxpatch_pft
              if (pft(i,j,m)<0 .or. pft(i,j,m)>numpft) then
                 write(6,*)'error: invalid PFT for i,j,m=',i,j,m,pft(i,j,m)
                 stop
              end if
           end do
           if (sumvec(i,j)>1.e-04 .and. landmask(i,j)==1) then
              write(6,*)'error: PFT cover ne 100 for i,j=',i,j
              do m=1,maxpatch_pft
                 write(6,*)'m= ',m,' pft= ',pft(i,j,m)
              end do
              write(6,*)'sumvec= ',sumvec(i,j)
              stop
           end if
        end do
     end do
  end if

  ! Error check: percent glacier, lake, wetland, urban sum must be less than 100

  do j=1,lsmlat
     do i=1,numlon(j)
        sumscl = pctlak(i,j)+pctwet(i,j)+pcturb(i,j)+pctgla(i,j)
        if (sumscl > 100.+1.e-04) then
           write(6,*)'error: PFT cover>100 for i,j=',i,j
           stop
        end if
     end do
  end do

  ! Check that urban parameterization is not yet implemented

  do j=1,lsmlat
     do i=1,numlon(j)
        if (pcturb(i,j) /= 0.) then
           write (6,*) 'error: urban parameterization not yet implemented'
           stop
        end if
     end do
  end do

  ! Make patch arrays, [veg] and [wt]:
  ! [veg] sets the PFT for each of the [maxpatch] patches on the 2d model grid.
  ! [wt]  sets the relative abundance of the PFT on the 2d model grid.
  ! Fill in PFTs for vegetated portion of grid cell. Fractional areas for
  ! these points [pctpft] pertain to "vegetated" area not to total grid area.
  ! So need to adjust them for fraction of grid that is vegetated.
  ! Next, fill in urban, lake, wetland, and glacier patches.

  veg(:,:,:) = 0
  wtxy(:,:,:)  = 0.
  do j=1,lsmlat
     do i=1,numlon(j)
        if (landmask(i,j) == 1) then
           sumscl = pcturb(i,j)+pctlak(i,j)+pctwet(i,j)+pctgla(i,j)
           do m = 1, maxpatch_pft
              if (do_DGVM) then
                 veg(i,j,m) = noveg !spinup initialization
                 wtxy(i,j,m) = 1.0/maxpatch_pft * (100.-sumscl)/100.
              else
                 veg(i,j,m) = pft(i,j,m)
                 wtxy(i,j,m) = pctpft(i,j,m) * (100.-sumscl)/10000.
              end if
           end do
           veg(i,j,npatch_urban) = noveg
           wtxy(i,j,npatch_urban) = pcturb(i,j)/100.
           veg(i,j,npatch_lake) = noveg
           wtxy(i,j,npatch_lake) = pctlak(i,j)/100.
           veg(i,j,npatch_wet) = noveg
           wtxy(i,j,npatch_wet) = pctwet(i,j)/100.
           veg(i,j,npatch_gla) = noveg
           wtxy(i,j,npatch_gla) = pctgla(i,j)/100.
        end if
     end do
  end do

  sumvec(:,:) = abs(sum(wtxy,dim=3)-1.)
  do j=1,lsmlat
     do i=1,numlon(j)
        if (sumvec(i,j) > 1.e-06 .and. landmask(i,j)==1) then
           write (6,*) 'error: WT > 1 occurs at i,j= ',i,j 
           stop
        endif
     end do
  end do

  ! Find total number of land grid cells [numland] and total number of 
  ! patches [numpatch] allowing for multiple subgrid patches in a grid cell

  npatch = 0
  nland = 0
  do j = 1, lsmlat
     do i = 1, numlon(j)
        if (landmask(i,j) == 1) then         
           nland = nland+1                               !land point number
           do m = 1, maxpatch                
              if (wtxy(i,j,m) > 0.)  npatch = npatch+1   !subgrid patch number
           end do
        end if
     end do
  end do

  write (6,*)
  write (6,*)' Surface Grid Characteristics'
  write (6,*)'   longitude points         = ',lsmlon
  write (6,*)'   latitude points          = ',lsmlat
  write (6,*)'   minimum longitude points = ',minval(numlon)
  write (6,*)'   maximum longitude points = ',maxval(numlon)
  write (6,*)'   land points on grid                    = ',nland
  write (6,*)'   total points including subgrid patches = ',npatch
  write (6,*)
       
  ! Allocate dynamic memory

  allocate (landvec%wtxy(nland,maxpatch))
  allocate (landvec%patch(nland,maxpatch))
  allocate (patchvec%mxy(npatch))
  allocate (patchvec%wtxy(npatch))

  ! Determine land vector and patch vector mapping components

  landvec%wtxy(:,:) = 0._r8
  patchvec%wtxy(:)  = 0._r8
  npatch = 0
  nland  = 0
  do j = 1, lsmlat
     do i = 1, numlon(j)
        if (landmask(i,j) == 1) then                 
           nland = nland+1                           
           do m = 1, maxpatch                           
              if (wtxy(i,j,m) > 0.) then                
                 npatch = npatch+1                      
                 landvec%patch(nland,m) = npatch        !land subgrid patch number
                 landvec%wtxy(nland,m)  = wtxy(i,j,m)   !land subgrid weights
                 patchvec%mxy(npatch)   = m             !patch subgrid index of lnd point
                 patchvec%wtxy(npatch)  = wtxy(i,j,m)   !patch weight
              end if
           end do
        end if
     end do
  end do

  ! Initialize land vector patches with zero weights to 
  ! first patch with non-zero weight

  do l = 1, nland
     n = 0
     do m = maxpatch, 1, -1
        if (landvec%wtxy(l,m) > 0.) n = m
     end do
     if (n == 0) then
        write (6,*) 'error: n = 0'
        stop
     end if
     do m = 1, maxpatch
        if (landvec%wtxy(l,m) == 0.) then
           landvec%patch(l,m) = landvec%patch(l,n)
        endif
     end do
  end do

  ! Error check: make sure weights sum to one for each land cell

  do l = 1, nland
     sumwts = 0
     do m = 1, maxpatch
        sumwts = sumwts + landvec%wtxy(l,m)
     end do
     if (abs(sumwts-1.) > 1.e-05) then
        write (6,*) 'error: weights do not sum to 1'
        write (6,*)' l = ',l,' sum = ',sumwts
        stop
     end if
  end do

  ! Determine number of gridcells, landunits, columns pfts for clm2.1

  numgrid = 0
  numland = 0
  numpfts = 0
  do j = 1, lsmlat
     do i = 1, numlon(j)
        if (landmask(i,j) == 1) then         
           numgrid = numgrid+1                              !gridcells
           do m = maxpatch_pft+1, maxpatch
              if (wtxy(i,j,m) > 0.) numland = numland + 1
           end do
           nvegpatch = 0
           do m = 1, maxpatch_pft                           
              if (wtxy(i,j,m) > 0.) nvegpatch = nvegpatch + 1
           end do
           if (nvegpatch > 0) numland = numland + 1         !landunits
           do m = 1, maxpatch                
              if (wtxy(i,j,m) > 0.)  numpfts = numpfts+1    !subgrid patches
           end do
        end if
     end do
  end do
  numcols = numpfts       

  write(6,*)'   clm2.1 gridcells = ',numgrid
  write(6,*)'   clm2.1 landunits = ',numland
  write(6,*)'   clm2.1 columns   = ',numpfts
  write(6,*)'   clm2.1 pfts      = ',numpfts
  write(6,*)

  return
end subroutine surfmap


